package com.fjhx.service.impl;

import cn.hutool.core.collection.ConcurrentHashSet;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.entity.ConnectConfig;
import com.fjhx.service.Operation;
import com.fjhx.utils.MyUtil;
import com.rfid.callBack.CallBack;
import com.rfid.uhf.controller.impl.ReaderR2000;
import com.rfid.uhf.service.ReaderR2000Service;
import com.rfid.uhf.service.impl.ReaderR2000ServiceImpl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class TcpOperation implements Operation {

    private final ReaderR2000Service service = new ReaderR2000ServiceImpl();
    private final List<ReaderR2000> readerR2000List = new ArrayList<>();
    private final Set<String> rfidSet = new ConcurrentHashSet<>();
    private final List<Thread> threadList = new ArrayList<>();
    private boolean monitorFlag = false;

    @Override
    public void distinctRead(String userId, String sessionId, List<ConnectConfig> connectConfigList, JSONObject data) {
        this.monitorFlag = true;

        for (ConnectConfig connectConfig : connectConfigList) {

            Thread thread = new Thread(() -> {
                ReaderR2000 reader = service.connect(
                        connectConfig.getIp(),
                        connectConfig.getPort(),
                        getCallBack(userId, sessionId, data)
                );
                service.beginInvV2(reader);
                readerR2000List.add(reader);
            });
            thread.start();
            threadList.add(thread);
        }

    }

    @Override
    public void close() {
        this.monitorFlag = false;
        for (ReaderR2000 readerR2000 : readerR2000List) {
            try {
                // 结束扫描
                service.stopInvV2(readerR2000);
                // 关闭连接
                service.disconnect(readerR2000);
            } catch (Exception e) {

            }
        }

        for (Thread thread : threadList) {
            try {
                thread.stop();
            } catch (Exception e) {
                MyUtil.errorLog("关闭线程失败");
                e.printStackTrace();
            }
        }

    }

    private CallBack.R2000 getCallBack(String userId, String sessionId, JSONObject data) {
        return new CallBack.R2000() {

            @Override
            public void readData(String rfid, String rssi, String antennaNo, String deviceNo, String direction, String communicationMode) {
                if (ObjectUtil.isEmpty(rfid) || rfidSet.contains(rfid)) {
                    return;
                }
                if (!monitorFlag) {
                    close();
                    return;
                }

                rfidSet.add(rfid);

                sendRfidDate(userId, sessionId, rfid, data);
            }

            public void serialPortException(String name, boolean result) {

            }

            public void heartBeat(String value, String deviceNo, String communicationMode) {
                // System.out.println("value " + value + " deviceNo " + deviceNo + " communicationMode " + communicationMode);
            }

            public void infraredTrigger(boolean direction, String value, String deviceNo, String communicationMode) {
                // System.out.println("方向 " + direction + " 红外值 " + value + " 设备号 " + deviceNo + " IP或串口号 " + communicationMode);
            }

            public void onceReadTagEnd(boolean arg0) {

            }

        };
    }

}
